function [ x, y, errest, time ] = zeta( sv, c, L, P, output )
% Main program: evaluate zeta function zeta( sv, c ) where c is parameter
% in map, s is vector of powers in zeta sum, L is number of levels and 
% P number of basis functions. Should give close to zero for s the dimension 
% associated with c: 
% [ x, y, e, t ] = zeta( 0.31850809575800, -20, 4, 1, 0 ) 
%       gives [ x = 2.7859e-10, y = -1.0229e-22, e = 0, t = 0.1800 ]
% and
% [ x, y, e, t ] = zeta( 0.48479829443816, -5,  4, 1, 0 )
%       gives [ x = 1.2824e-06, y = -1.1619e-20, e = 0, t = 0.1800 ]
% while with higher accuracy
% [ x, y, e, t ] = zeta( 0.31850809575800, -20, 8, 5, 0 )
%		gives [ x = -1.1571e-14, y = -3.3773e-17, e = 0, t = 7.2900 ]
% and
% [ x, y, e, t ] = zeta( 0.48479829443816, -5,  8, 5, 0 )
%		gives [ x = -6.5302e-16, y = -2.7861e-17, e = 0, t = 7.2900 ]
%
% Returns an error estimate (not yet implemented) and the CPU time required if requested.
% Can be used to find dimension for any given c by Matlab's builtin fzero as follows:
% [ d, x, exit, output ] = fzero( @zeta, [ 0, 2 ], [ ], c, L, P )
% returns the dimension in d, real part of zeta value in x, exit status and iterations
% for a given value of c. Here [ 0, 2 ] is the starting interval and [ ] is an options
% array for fzero which I don't know how to set profitably. The 2 sign tells Matlab that
% "zeta" is the name of a function.
% For example, the loop
% for L = 3:7
% for P = 1:4
% [ d, x, exit, output ] = fzero( @zeta, [ 0, 2 ], [ ], c, L, P );
% a( L - 2, P ) = d;
% produces a =
%
%         P = 1              P = 2              P = 3              P = 4
%   L = 3 0.48478348721389   0.48479099537140   0.48479829294510   0.48479829372883
%   L = 4 0.48479741542081   0.48479784089445   0.48479829442433   0.48479829443523
%   L = 5 0.48479823885691   0.48479826617332   0.48479829443815   0.48479829443815
%   L = 6 0.48479829098609   0.48479829267778  *0.48479829443816*  0.48479829443816
%   L = 7 0.48479829422308   0.48479829432851   0.48479829443816   0.48479829443816
%
% which converges nicely to the correct value 0.48479829443816 already with L = 6, P = 3
% (requiring the determinant of a 192 x 192 matrix). The computation took CPU times t = 
%
%         P = 1              P = 2              P = 3              P = 4
%          1.0300             1.4300             1.5600             1.7600
%          1.8200             2.8300             3.0600             3.4900
%          3.6300             5.6400             6.1700             7.0400
%          7.2400            11.2500           *12.4500*           14.3400
%         14.7800            22.9100            25.1600            30.7200


 % Set useful input values for unspecified arguments.

 % Parameter c in z^2 + c map.
 if( nargin < 1 )
  c = -5.0;
 end

 % Parameter s in zeta function.
 if( nargin < 2 )
  sv = 0.48479829443816;
 end

 % Number of levels to iterate around Julia set.
 if( nargin < 3 )
  L = 5;
 end

 % Number of intervals = number of blocks per dimension of transfer matrix.
 N = 2^L;
 
 % Number of powers per interval = block size of transfer matrix.
 if( nargin < 4 )
  P = 2;
 end
 
 % Output control: 0 is no output, more output for higher values.
 if( nargin < 5 )
  output = 0;
 end
 
 % Don't compute unwanted output arguments.
 if( nargout > 2 )
  t0 = cputime;
 end
 
 if( nargout > 1 )
  errest = 0.0;
 end
 
 % Here we should build a quadrature formula that gives desired accuracy
 % depending on the physical parameters c and s. In the meantime, we use
 % Tenth order quadrature formula on unit disk from AS p. 892.

 % Order of accuracy for numerical quadrature---later tie to P.
 q = 10;
 nq = 21;
 zq = ones( nq, 1 );
 wq = ones( nq, 1 );

 zq( 1 ) = 0.0 + 0.0*i;
 wq( 1 ) = 1.0 / 9.0;

 
 ibeg = 1;
 ss = sqrt( 0.1 * ( 6.0 - sqrt( 6.0 ) ) ); 
 ww = ( 16.0 + sqrt( 6.0 ) ) / 360.0;
 for iq = 1 : 10
  zq( ibeg + iq ) = ss * ( exp( 0.2 * pi * iq * i ) );
  wq( ibeg + iq ) = ww;
 end

 ibeg = 11;
 ss = sqrt( 0.1 * ( 6.0 + sqrt( 6.0 ) ) ); 
 ww = ( 16.0 - sqrt( 6.0 ) ) / 360.0;
 for iq = 1 : 10
  zq( ibeg + iq ) = ss * ( exp( 0.2 * pi * iq * i ) );
  wq( ibeg + iq ) = ww;
 end

 wq = pi * wq;
 % This completes the quadrature-building section. We may want to test it:

 if( output > 1 )
  % Test numerical integration by integrating powers over unit disk.
  for k = 0 : nq
   sum = 0.0 + 0.0*i;
   for iq = 1 : nq
    sum = sum + wq( iq ) * ( zq( iq )' )^k * zq( iq )^k;
   end
   sum = ( ( k + 1 ) / PI ) * sum;
   fprintf( 1, 'sum( %d ) = %f + %f i should be 1 + 0i\n', k, real( sum ), imag( sum ) );
  end
 end

 % Initialize and set arrays to maximum sizes required.
 
 d = 0.5 * ( 1.0 + sqrt( 1 - 4.0 * c ) );
% d

 emin = zeros( N, 1 );
 emax = zeros( N, 1 );
 indices = zeros( N, L );

 emin( 1 : 2 ) = sort( [ sqrt(-c-d) - 0.25 , -d - 0.25 ] );
 emax( 1 : 2 ) = sort( [ d + 0.25 , -sqrt(-c-d) + 0.25 ] );
 indices( 1 : 2 ) = [ 1 ; 2 ];

 N = 2;

 % Iterate
 for level = 2 : L
  
  N2 = 2 * N;
  tmp( 1 : N ) = -1 * sqrt( emin( 1 : N ) - c * ones( N, 1 ) );
  tmp( N + 1 : N2 ) = sqrt( emin( 1 : N ) - c * ones( N, 1 ) );
  emin( 1 : N2 ) = tmp( 1 : N2 );
 
  tmp( 1 : N ) = -1 * sqrt( emax( 1 : N ) - c * ones( N, 1 ) );
  tmp( N + 1 : N2 ) = sqrt( emax( 1 : N ) - c * ones( N, 1 ) );
  emax( 1 : N2 ) = tmp( 1 : N2 );

  tmpi( 1 : N , 2 : level ) = indices( 1 : N , 1 : level - 1 );
  tmpi( N + 1 : N2 , 2 : level ) = indices( 1 : N , 1 : level - 1 );
  tmpi( 1 : N , 1 ) = 1 * ones( N, 1 );
  tmpi( N + 1 : N2 , 1 ) = 2 * ones( N, 1 );
  indices( 1 : N2, 1 : level ) = tmpi( 1 : N2, 1 : level );

  N = N2;

  % Reorder emin and emax as necessary to have emin(ib) < emax(ib) foreach ib.
  for ib = 1 : N
   if( emin( ib ) > emax( ib ) )
    swap = emax( ib );
    emax( ib ) = emin( ib );
    emin( ib ) = swap;
   end % if( emin( ib ) ...
  end % End for ib = ...

  % We may want to check inclusions at this stage and print any errors that occur.
  if( output > 0 )
   for ib = 1 : N
    fprintf( 1, 'level-%d interval %d is [%f, %f]', level, ib, emin( ib ), emax( ib ) );
    fprintf( 1, ' has history ' );
    for jb = 1 : level
      fprintf( 1, '%d', indices( ib, jb ) );
    end
    fprintf( 1, '\n' );
  
   end % for ib = ...
 
   for b = 1 : 2
    fprintf( 1, 'g %d yields intervals\n', b );
    for ib = 1 : N
     gbmin = (-1)^b * sqrt( emin( ib ) - c );
     gbmax = (-1)^b * sqrt( emax( ib ) - c );
 
     if( gbmin > gbmax )
      swap = gbmin;
      gbmin = gbmax;
      gbmax = swap;
     end % if( gbmin ...

     fprintf( 1, '%d [%f, %f] inside history ', ib, gbmin, gbmax );
     % Apply sigma to obtain result of mapping by g.
     sg( 1 ) = b;
     sg( 2 : level )  = indices( ib, 1: level - 1 );

     for jb = 1 : level
      fprintf( 1, '%d', sg( jb ) );
     end % for jb = ...

     % Convert history indices into row of indices matrix.
     jb = 1;
     m = 1;
     for l = level : -1 : 1
      jb = jb + ( sg( l ) - 1 ) * m;
      m = 2 * m;
     end
     fprintf( 1, ' of interval %d ...', jb );

     % Now we'll actually check that interval ib is contained in jb and print messages.
     if( ( gbmin >= emin( jb ) ) & ( gbmax <= emax( jb ) ) )
      fprintf( 1, 'OK\n' );
     else
      fprintf( 1, 'OOPS\n' );
     end % if( ( gbmin ...

    end % for ib = ...
   end % for b = ...
  end % if( output ...

 end % End for level = ...

 % Now we have the endpts in pairs: form matrix of transfer operator
 % in basis consisting of complex powers centered and scaled to each interval.

 % Number of intervals is half the number of endpts.
 radius = 0.5 * ( emax - emin );
 center = emin + radius;

 if( output > 1 )
  fprintf( 1, 'N = %d\n\n', N );
  fprintf( 1, 'left endpts = %f %f %f %f\n', emin );
  fprintf( 1, 'right endpts = %f %f %f %f\n', emax );
  fprintf( 1, 'center = %f %f %f %f \n', center );
  fprintf( 1, 'radius = %f %f %f %f\n', radius );
 end

 [ m, n ] = size( sv );
 zz = zeros( m, n );

 for is = 1 : m
 for js = 1 : n
  s = sv( is, js );
  
 clear Lm a;
 Lm = sparse( N * P, N * P );
 a = zeros( P, P );

 for ib = 1 : N
  for b = 1 : 2

   % Apply sigma to obtain result of mapping by g.
   sg( 1 ) = b;
   sg( 2 : L )  = indices( ib, 1: L - 1 );

   % Convert history indices into row of indices matrix.
   jb = 1;
   m = 1;
   for l = L : -1 : 1
    jb = jb + ( sg( l ) - 1 ) * m;
    m = 2 * m;
   end

   %Form PxP block (ib, jb) by integrating basis functions against transformed
   % basis functions times power of square root over circle D_ib.
   %   a = block( ib, jb, b, P, L, N, center, radius, indices, c, s );
   % Build nonzero PxP block at location ( ib, jb ) in block matrix, by integrating
   % f_ik* vs L_b f_jl over circle D_ib for basis functions 1 <= k,l <= P on circles:
   %		
   %	f_ik(z) = ( 1/radius_ib) sqrt( (k+1) / pi ) ( ( z - center_ib ) / radius_ib )^k
   %
   % Here f_jl is transformed by L_b to yield
   % 
   %	L_b f_jl(z) = [ g_b'(z) ]^s f_jl( g_b(z) ).
   %
   % The square brackets indicate that we always take the positive sign so 
   %
   %	[ g_b'(z) ] = 1 / ( 2 * sqrt( z - c ) ).
     
   ri = radius( ib );
   ci = center( ib );
   rj = radius( jb );
   cj = center( jb );
  
   % Shift and scale quadrature points to D_i
   zqi = ci * ones( size( zq ) ) + ri * zq;
 
   % Integrate f_ik* vs L_b f_jl over D_i and accumulate in a( k, l )
   a = zeros( P, P );
   zik = zeros( P, 1 );
   zbjl = zeros( P, 1 );
 
   for iq = 1 : nq
    z = zqi( iq );
    w = wq( iq );
 
    % This introduces wasteful extra arithmetic since we are undoing what we just did.
    zi = conj( z - ci ) / ri;
    % This does less arithmetic and produces different (maybe better) results.
    zi = conj( zq( iq ) );
 
    gs = ( 0.5 / sqrt( z - c ) )^s;
    wgs = w * gs;
 
    zb =(-1)^b * sqrt( z - c );
    zbj = conj( zb - cj ) / rj;
 
    % Tabulate powers. Later we should just do this once for all blocks to save time.
    zik( 1 ) = 1.0;
    for k = 1 : P - 1
     zik( k + 1 )  = zik( k ) * zi;
    end
 
    zbjl( 1 ) = 1.0;
    for l = 1 : P - 1
     zbjl( l + 1 )  = zbjl( l ) * zbj;
    end
 
    a = a + wgs * zik * zbjl';
   end
 
   % Perform final scaling by all the factors that don't depend on anything else.
   a = ri * a / ( rj * pi );
  
   sqi = zeros( P, 1 );
   for i = 1 : P
    sqi( i ) = sqrt( i );
   end
   for k = 1 : P
    a( k, 1 : P ) = sqi( k ) * a( k, 1 : P );
   end
   for l = 1 : P
    a( 1 : P, l ) = sqi( l ) * a( 1 : P, l );
   end

%  ib, jb, a
 
   Lm( ( ib - 1 ) * P + 1 : ib * P, ( jb - 1 ) * P + 1 : jb * P ) = a;
 
  end % End for b = ...
 end % End for ib = ...

 Lm = speye( N * P ) - Lm;
 
 % Calculate zeta function det( I - Lm ).
 zz(is, js) = det( Lm );
 end % for is = ...
 end % for js = ...

 x = real( zz );
 y = imag( zz );
 
 if( nargout > 2 )
  time = cputime - t0;
 end
